use calibre_db::data::error::DataStoreError;
use poem::{error::ResponseError, http::StatusCode, Body, Response};
use tracing::error;
use uuid::Uuid;

#[derive(Debug, thiserror::Error)]
#[error("sqlite error")]
pub struct SqliteError(pub DataStoreError);

impl From<DataStoreError> for SqliteError {
    fn from(item: DataStoreError) -> Self {
        SqliteError(item)
    }
}

impl ResponseError for SqliteError {
    fn status(&self) -> StatusCode {
        match &self.0 {
            DataStoreError::NoResults(_) => StatusCode::NOT_FOUND,
            _ => StatusCode::INTERNAL_SERVER_ERROR,
        }
    }

    fn as_response(&self) -> Response {
        let id = Uuid::new_v4();
        let internal_msg = format!("{:?}", self);
        let external_msg = match &self.0 {
            DataStoreError::NoResults(_) => "item not found",
            _ => "internal server error",
        };
        error!("{id}: {internal_msg}");

        let body = Body::from_json(serde_json::json!({
            "id": id.to_string(),
            "message": external_msg,
        }))
        .unwrap();
        Response::builder().status(self.status()).body(body)
    }
}
