use std::sync::Arc;

use calibre_db::data::pagination::SortOrder;
use poem::{
    handler,
    web::{Data, Html, Path},
};

use crate::app_state::AppState;

use super::paginated;

#[handler]
pub async fn handler_init(state: Data<&Arc<AppState>>) -> Result<Html<String>, poem::Error> {
    series(&state, None, &SortOrder::ASC)
}

#[handler]
pub async fn handler(
    Path((cursor, sort_order)): Path<(String, SortOrder)>,
    state: Data<&Arc<AppState>>,
) -> Result<Html<String>, poem::Error> {
    series(&state, Some(&cursor), &sort_order)
}

fn series(
    state: &Arc<AppState>,
    cursor: Option<&str>,
    sort_order: &SortOrder,
) -> Result<Html<String>, poem::Error> {
    paginated::render(
        "series",
        || state.calibre.series(25, cursor, sort_order),
        |series| series.sort.clone(),
        |cursor| state.calibre.has_previous_series(cursor),
        |cursor| state.calibre.has_more_series(cursor),
    )
}
